; ##############################################################################
; ##############################################################################
; ##############################################################################
;																				
; Timer 0+1+2 berlauf-Int aktivieren (Timer/Counter Interrupt Mask)			
;																				
TIMER_AKTIVIEREN:
	ldi temp,(0<<OCIE2)|(0<<TOIE2)|(0<<TICIE1)|(0<<OCIE1A)|(0<<OCIE1B)|(0<<TOIE1)|(1<<TOIE0)
	out TIMSK,temp       
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; TIMER0 (8Bit) Vorteiler														
;																				
; CS02 CS01 CS00 	Description													
;	0 	0 	0 		No clock source (Timer/Counter stopped).					
;	0 	0 	1 		clkI/O/		(No prescaling)									
;	0 	1 	0 		clkI/O/8 	(From prescaler)								
; *	0 	1 	1 		clkI/O/64 	(From prescaler)								
; 	1 	0 	0 		clkI/O/256 	(From prescaler)								
; 	1 	0 	1 		clkI/O/1024 (From prescaler)								
;	1 	1 	0 		External clock source on T0 pin. Clock on falling edge.		
;	1 	1 	1 		External clock source on T0 pin. Clock on rising edge.		
TIMER0_INITIALISIERUNG_ms:
.equ TIMERWERT0	=256-20	;(2 hoch 8  minus...)		bei 1MHz + Vorteiler   64 => 0.001s 	
	ldi temp,(0<<CS02)|(1<<CS01)|(1<<CS00)
	OUT TCCR0,temp

	ldi temp, TIMERWERT0	; TIMER0 Startwert		
	out TCNT0,temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TIM0_OVF:
	push TEMP		    ; Sichern von "TEMP" im Stack
	in	 TEMP,SREG		; Einlesen des SREG 
	push TEMP	    	; Schreiben von  SREG  im Stack (KOPIE)
	push TEMP1
;-------------------------------------------------------------------------------
	ldi temp, TIMERWERT0; TIMER0 Startwert					
	out TCNT0,temp
;-------------------------------------------------------------------------------
; Tasten																		
	LDS temp,(adr_TASTE)
	tst temp					; Null ?	
	brne TIM0_OVF_RUN			; nein		
	in temp,PIND
	com temp
	STS(adr_TASTE),temp			; save		
	tst temp					; Null ?	
	breq TIM0_OVF_RUN			; ja		

	cpi temp,1					; Taste ?	
	breq TIM0_OVF_RUN			; ja		
	cpi temp,2					; Taste ?	
	breq TIM0_OVF_RUN			; ja		
	cpi temp,4					; Taste ?	
	breq TIM0_OVF_RUN			; ja		
	cpi temp,64					; Taste ?	
	breq TIM0_OVF_RUN			; ja		
	cpi temp,128				; Taste ?	
	breq TIM0_OVF_RUN			; ja		
	rjmp TIM0_RESET
;-------------------------------------------------------------------------------
; 1ms																			
TIM0_OVF_RUN:
	LDS temp,(adr_VT1)
	inc temp
	cpi temp,(9+1)	; *9+1	
	brsh TIM0_10ms
	STS(adr_VT1),temp
	rjmp TIM0_FERTIG
;-------------------------------------------------------------------------------
; 10ms																			
TIM0_10ms:
	STS(adr_VT1),NULL
	LDS temp,(adr_VT2)
	inc temp
	cpi temp,(9+1)	; *9+1	
	brsh TIM0_100ms
	STS(adr_VT2),temp
	rjmp TIM0_FERTIG
;-------------------------------------------------------------------------------
; 100ms																			
TIM0_100ms:
	STS(adr_VT2),NULL
	LDS temp,(adr_VT3)
	inc temp
	cpi temp,(9+1)
	brsh TIM0_1s
	STS(adr_VT3),temp
	rjmp TIM0_FERTIG
;-------------------------------------------------------------------------------
; 1s																			
TIM0_1s:
	STS(adr_VT3),NULL
	rcall UHR
;-------------------------------------------------------------------------------
TIM0_FERTIG:
	pop TEMP1
	pop	TEMP					; LESEN von SREG vom STACK (KOPIE)	
	out	SREG,TEMP				; Wiederherstellen von SREG			
	pop	TEMP					; Wiederherstellen von "TEMP" 		
reti
;-------------------------------------------------------------------------------
TIM0_RESET:
	GRUEN_ON
	GELB_ON
	ROT_ON
	BLAU_ON
	BEEP_ON
	rjmp TIM0_RESET
; ##############################################################################
; ##############################################################################
; ##############################################################################
UHR:
	STS (adr_FREIGABE_ss),EINS	; Freigabe							

	LDS temp,(adr_ss)
	inc temp
	cpi temp,60
	brsh UHR_mm
	STS (adr_ss),temp
	ret
;-------------------------------------------------------------------------------
UHR_mm:
	STS (adr_FREIGABE_mm),EINS	; Freigabe							

	STS (adr_ss),NULL
	LDS temp,(adr_mm)
	inc temp
	cpi temp,60
	brsh UHR_hh
	STS (adr_mm),temp
	ret
;-------------------------------------------------------------------------------
UHR_hh:
	STS (adr_FREIGABE_hh),EINS	; Freigabe							

	STS (adr_mm),NULL			; in SRAM speichern					
	LDS temp,(adr_hh)			; aus SRAM laden					
	inc temp
	cpi temp,24
	brsh UHR_WOCHENTAG
	STS (adr_hh),temp			; in SRAM speichern					
	ret
;-------------------------------------------------------------------------------
; WOCHENTAG																		
UHR_WOCHENTAG:	
	STS (adr_FREIGABE_TAG),EINS	; Freigabe							

	LDS temp,(adr_WOCHENTAG)
	inc temp
	cpi temp,7+1
	brlo UHR_WOCHENTAG_w
	ldi temp,1
UHR_WOCHENTAG_w:
	STS(adr_WOCHENTAG),temp
;-------------------------------------------------------------------------------
; TAG																			
UHR_TAG:
	STS (adr_hh),NULL

	LDS temp,(adr_TAG)
	inc temp
	cpi temp,31+1 
	brsh UHR_MONAT
	STS (adr_TAG),temp
	ret
;-------------------------------------------------------------------------------
; MONAT																			
UHR_MONAT:
	STS (adr_TAG),EINS

	LDS temp,(adr_MONAT)
	inc temp
	STS (adr_MONAT),temp
	cpi temp,(12+1)
	brsh UHR_JAHR
	ret
;-------------------------------------------------------------------------------
; JAHR					
UHR_JAHR:
	STS (adr_MONAT),EINS
	LDS temp,(adr_JAHR)
	inc temp
	STS (adr_JAHR),temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
