; ##############################################################################
; ##############################################################################
; ##############################################################################
; TWI MASTER EINSTELLUNGEN											   			
TWI_INI:

.equ TWI_BIT_RATE			= 0		; 2...35...100	 Bit-Rate						
.equ TWI_PRESCALER			= 0		; 0				 Vorteiler: 0=1 1=4 2=16 3=64	

	; SCL(Hz) = CPU(Hz) / (16 + 2 * TWBR * 4^TWPS)	
	; Beispiel:										
	; 100.000 = 16.000.000 / (16 + 2 * 18 * 4^1)	

	; TWI TWBR 				Bit RATE Register	
	ldi temp, TWI_BIT_RATE
	out TWBR, temp
	; TWI TWSR 				STATUS-Register		
	ldi temp, TWI_PRESCALER
	out TWSR, temp
	; TWI TWAR (nur SLAVE) 	Adress-Register		
	clr temp
	out TWAR,temp
	; TWI TWCR 				Control-REGISTER	
	ldi temp, 1<<TWINT|1<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|1<<TWEN|0<<TWIE
	out TWCR,temp

ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TWI_SCAN_CHECK:
	rcall TFT_CLEAR
	rcall FARBE_WEISS	; FARBE	
;-------------------------------------------------------------------------------
	rcall ZEILE_2
	rcall ASCII_T
	rcall ASCII_F
	rcall ASCII_T
	rcall ASCII_SPACE4
	ldi temp1,(TWI_ADR_TFT)
	rcall ZAHL_TEMP1
	ldi temp1,(TWI_ADR_TFT)
	rcall TWI_MT_START_TEMP1
	brcc TWI_CHECK_ERROR
	rcall TEXT_OK		; OK	
	rcall TFT_PAKET_SENDEN
;-------------------------------------------------------------------------------
	rcall ZEILE_4
	rcall ASCII_R
	rcall ASCII_T
	rcall ASCII_C
	rcall ASCII_SPACE4
	ldi temp1,(TWI_ADR_RTC)
	rcall ZAHL_TEMP1
	ldi temp1,(TWI_ADR_RTC)
	rcall TWI_MT_START_TEMP1
	brcc TWI_CHECK_ERROR
	rcall TEXT_OK		; OK	
	rcall TFT_PAKET_SENDEN
;-------------------------------------------------------------------------------
	rcall ZEILE_6
	rcall ASCII_E
	rcall ASCII_E
	rcall ASCII_P
	rcall ASCII_SPACE4
	ldi temp1,(TWI_ADR_EEPROM)
	rcall ZAHL_TEMP1
	ldi temp1,(TWI_ADR_EEPROM)
	rcall TWI_MT_START_TEMP1
	brcc TWI_CHECK_ERROR
	rcall TEXT_OK		; OK	
	rcall TFT_PAKET_SENDEN
;-------------------------------------------------------------------------------
	rcall ZEILE_10
	ldi temp1,0
TWI_SCAN_s:
	push temp1
	rcall TWI_MT_START_TEMP1
	rcall WAIT_10ms
	rcall TWI_STOP
	pop temp1
	brcc TWI_SCAN_w
	push temp1	
	rcall ZAHL_TEMP1
	rcall TFT_PAKET_SENDEN
	pop temp1
TWI_SCAN_w:
	inc temp1				; +1	
	cpi temp1,128
	brne TWI_SCAN_s
	rcall TWI_STOP
	rjmp TFT_PAKET_SENDEN
;-------------------------------------------------------------------------------
TWI_CHECK_ERROR:
	rcall FARBE_ROT	; FARBE
	BEEP_ON
	rcall TEXT_ERROR	
	rcall TFT_PAKET_SENDEN
;-------------------------------------------------------------------------------
TWI_CHECK_ERROR_s:
	rjmp TWI_CHECK_ERROR_s
; ##############################################################################
; ##############################################################################
; ##############################################################################
;																			   	
; START-ROUTINE	(MASTER TRANSMITTER)										   	
;																			   	
; INPUT:	temp1 <= ADRESSE SLAVE											   	
; OUT:		FEHLER_CODE/CARRY													
;																			   	
TWI_MT_START_TEMP1:
	clr ERROR									; FEHLER lschen					
	ldi temp, (1<<TWINT)|(1<<TWSTA)|(1<<TWEN)	; START								
	out TWCR, temp	
	rcall TWI_TWINT_wait						; warten, bis TWINT wieder = 1						
	in temp,TWSR								; Check TWSR						
	andi temp, 0xF8
	cpi temp,  0x08								; 0x08 START erfolgreich			
	breq TWI_MT_START_w
	cpi temp,  0x10								; 0x10 (repeated START)				
	breq TWI_MT_START_w
;-------------------------------------------------------------------------------
	; ERROR		
	mov ERROR,VOLL
	CLC 										; CLEAR CARRY
	ret
;-------------------------------------------------------------------------------
TWI_MT_START_w:
	; Load SLA + "W"  (W=0) 			
	LSL  temp1									; 1 x links
	out TWDR, temp1
	; senden - Clear TWINT 				
	ldi temp, (1<<TWINT) | (1<<TWEN)							
	out TWCR, temp
	; warten, bis TWINT wieder = 1		
	rcall TWI_TWINT_wait									
	; Check TWSR						
	in temp,TWSR												
	andi temp, 0xF8
	cpi temp,  0x18								; 0x18		
	breq TWI_MT_START_OK
;-------------------------------------------------------------------------------
	; ERROR		
	mov ERROR,VOLL
	CLC 										; CLEAR CARRY
	ret
;-------------------------------------------------------------------------------
TWI_MT_START_OK:
	SEC 										; SET Carry
ret
; #############################################################################
; #############################################################################
; #############################################################################
; 																			   
; Ein Zeichen zum SLAVE zu senden										   	   
; 																			   
; INPUT:	temp1 <= DATEN													   
; OUT:		ERROR_CODE/CARRY													
; 																			   
TWI_MT_DATA_TEMP1:
	tst ERROR		; FEHLERPRFUNG		
	breq TWI_MT_DATA_w
	CLC 			; CLEAR CARRY		
	ret									
;-------------------------------------------------------------------------------
TWI_MT_DATA_w:
	; 	Load DATA						
	out TWDR, temp1							
	; senden - Clear TWINT 				
	ldi temp, (1<<TWINT) | (1<<TWEN)							
	out TWCR, temp
	; warten, bis TWINT wieder = 1		
	rcall TWI_TWINT_wait									
	; Check TWSR						
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x28	; 0x28				
	breq TWI_MT_DATA_OK	
;-------------------------------------------------------------------------------
	; ERROR								
	mov ERROR,VOLL
	CLC 			; CLEAR CARRY		
	ret
;-------------------------------------------------------------------------------
TWI_MT_DATA_OK:
	SEC 			; SET Carry			
ret
; #############################################################################
; #############################################################################
; #############################################################################
;																			   
; TWI/I2C START-ROUTINE														   
;																			   
; INP:	temp1 <= ADRESSE SLAVE											   		
; OUT:		ERROR_CODE/CARRY													
;																			   	
TWI_MR_START_TEMP1:
	WDR
	clr ERROR		; FEHLER lschen					
	; START 											
	ldi temp, (1<<TWINT | 1<<TWEA | 1<<TWSTA | 1<<TWEN )
	out TWCR, temp	
	; warten, bis TWINT wieder = 1						
	rcall TWI_TWINT_wait
	; Check TWSR 										
	in temp,TWSR		
	andi temp, 0xF8
	cpi temp,  0x08 								; 0x08 START erfolgreich
	breq TWI_MASTER_RECEIVER_START_TEMP1_w
	cpi temp,  0x10									; 0x10 (repeated START)
	breq TWI_MASTER_RECEIVER_START_TEMP1_w
;-------------------------------------------------------------------------------
	; ERROR												
	mov ERROR,VOLL
	CLC ; CLEAR CARRY
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_START_TEMP1_w:
	; Load SLA +"R" (R=1)								
	LSL  temp1	; 1 x links
	ori temp1, 0b00000001 
	out TWDR, temp1
	; senden - Clear TWINT 								
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )
	out TWCR, temp
	; warten, bis TWINT wieder = 1						
	rcall TWI_TWINT_wait
	; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x40 	;SLA+R gesendet; ACK empfangen
	breq TWI_MASTER_RECEIVER_START_TEMP1_OK
;-------------------------------------------------------------------------------
	; ERROR						
	mov ERROR,VOLL
	CLC ; CLEAR CARRY
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_START_TEMP1_OK:
	SEC ; SET Carry
ret
; #############################################################################
; #############################################################################
; #############################################################################
; Mehrere BYTES sollen empfangen werden											
;																				
; OUT:		temp1/ERROR_CODE/CARRY												
;																				
TWI_MR_DATA_TEMP1:
	; Fehlerprfung										
	tst ERROR
	breq TWI_MASTER_RECEIVER_DATA_TEMP1_w
	CLC ; CLEAR CARRY
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_DATA_TEMP1_w:
	; senden "ACK" (TWEA=1) fr mehrere Daten-Bytes 	
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )
	out TWCR, temp
	; TWINT abfragen									
	rcall TWI_TWINT_wait
	; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x50	; Data Byte empfangen- ACK zurckgesendet
	breq TWI_MASTER_RECEIVER_DATA_TEMP1_OK
;-------------------------------------------------------------------------------
	; ERROR
	mov ERROR,VOLL
	CLC ; CLEAR CARRY
	clr temp1
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_DATA_TEMP1_OK:
	; DATEN einlesen							
	in temp1, TWDR						
	SEC ; SET Carry
ret
; #############################################################################
; #############################################################################
; #############################################################################
; Ein BYTE oder letztes BYTE soll empfangen werden 								
;																				
; OUT:		temp1/ERROR_CODE/CARRY												
;																				
TWI_MR_LAST_DATA_TEMP1:
TWI_MASTER_RECEIVER_DATA_LAST_TEMP1x:
	; Fehlerprfung										
	tst ERROR
	breq TWI_MASTER_RECEIVER_DATA_LAST_TEMP1_w
	CLC ; CLEAR CARRY
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_DATA_LAST_TEMP1_w:
	; senden "NOT ACK" (TWEA=0) fr letztes bzw fr ein Daten-Byte
	ldi temp, (1<<TWINT | 0<<TWEA  | 1<<TWEN )
	out TWCR, temp
	; TWINT abfragen									
	rcall TWI_TWINT_wait
	; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi  temp, 0x58	; letztes Data Byte empfangen-NOT ACK zurckgesendet
	breq TWI_MASTER_RECEIVER_DATA_LAST_TEMP1_OK	; ERROR
;-------------------------------------------------------------------------------
	; ERROR
	mov ERROR,VOLL
	CLC ; CLEAR CARRY
	clr temp1
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_DATA_LAST_TEMP1_OK:
	; DATEN einlesen		
	in temp1, TWDR
	SEC ; SET Carry
ret
; #############################################################################
; #############################################################################
; #############################################################################
; 																			   
; STOP- Routine																   
; 																			   
TWI_STOP:
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)|0<<TWIE	; STOP								
	out TWCR, temp	
	; kurze Pause	
	rcall wait_50us	
ret
; #############################################################################
; #############################################################################
; #############################################################################
TWI_AUS:
	ldi temp, 1<<TWINT|0<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|0<<TWEN|0<<TWIE
	out TWCR,temp
ret
; #############################################################################
; #############################################################################
; #############################################################################
; 																			   
; ERROR- Routine															   
; 																			   
TWI_ERROR:
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	out TWCR, temp
	rcall wait_100ms	
	rcall TWI_AUS
ret
; #############################################################################
; #############################################################################
; #############################################################################
; 																			   
; warten, bis das TWINT Flag gesetzt ist									   
; 																			   
TWI_TWINT_wait:
	GELB_ON
	in temp, TWCR
	sbrs temp, TWINT	; nchsten Befehl berspringen, wenn TWINT FLAG  gesetzt ist
	rjmp TWI_TWINT_wait	
	GELB_OFF
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################

